/* packet-mapem.c
 * Routines for MAPEM packet dissection
 *
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <glib.h>
#include <epan/packet.h>
#include <epan/asn1.h>
#include <epan/dissectors/packet-per.h>
#include <epan/prefs.h>

#include <stdio.h>
#include <string.h>

#include "packet-mapem.h"

#define PNAME  "MAPEM"
#define PSNAME "MAPEM"
#define PFNAME "mapem"
#define MAPEM_PORT 2003    /* BTP port */

void proto_reg_handoff_mapem(void);

/* Initialize the protocol and registered fields */
static int proto_mapem = -1;

/* Dissector tables */
static int global_mapem_port = MAPEM_PORT;
/* Used to identify MessageFrame */
static guint8 Ref_DSRCmsgID;
/* Used to identify RegionalExtension */
static guint16 Ref_RegionId;

typedef enum _RefDSRCmsgID_enum {
  mapData                     = 18,
  signalPhaseAndTimingMessage = 19,
  signalRequestMessage        = 20,
  signalStatusMessage         = 30
} refDSRCmsgID_enum;

typedef enum _RefRegionID_enum {
  noRegion = 0,
  addGrpA  = 1,
  addGrpB  = 2,
  addGrpC  = 3
} RefRegionId_enum;

#include "packet-mapem-hf.c"

/* Initialize the subtree pointers */
static int ett_mapem = -1;

static dissector_table_t dissect_messageframe_pdu_type_table;
static dissector_table_t dissect_regionalextension_pdu_type_table;

static int dissect_messageframe_pdu_type(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_);
static int dissect_regionalextension_pdu_type(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_);
static int dissect_xxx_Container(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_);

#include "packet-mapem-ett.c"

#include "packet-mapem-fn.c"

#if defined(BUILD_WINDOWS)
guint32
dissect_per_UTF8String(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, int min_len, int max_len, gboolean has_extension)
{
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index, min_len, max_len, has_extension, NULL);
  return offset;
}
#else
extern guint32 dissect_per_UTF8String(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, int min_len, int max_len, gboolean has_extension);
#endif

static int
dissect_xxx_Container(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  printf("### dissect_XXX_container: Unsupported\n");
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 32767U, NULL, FALSE);

  return offset;
}

static int dissect_messageframe_pdu_type(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_)
{
  return (dissector_try_uint_new(dissect_messageframe_pdu_type_table, Ref_DSRCmsgID, tvb, pinfo, tree, FALSE, NULL)) ? tvb_captured_length(tvb) : 0;
}

static int dissect_regionalextension_pdu_type(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_)
{
  return (dissector_try_uint_new(dissect_regionalextension_pdu_type_table, Ref_RegionId, tvb, pinfo, tree, FALSE, NULL)) ? tvb_captured_length(tvb) : 0;
}

static void
dissect_mapem(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
    proto_item      *mapem_item = NULL;
    proto_tree      *mapem_tree = NULL;

    /* make entry in the Protocol column on summary display */
    col_set_str(pinfo->cinfo, COL_PROTOCOL, PNAME);

    /* Clear out stuff in the info column */
    col_clear(pinfo->cinfo,COL_INFO);

    /* create the mapem protocol tree */
    if (tree) {
        mapem_item = proto_tree_add_item(tree, proto_mapem, tvb, 0, -1, FALSE);
        mapem_tree = proto_item_add_subtree(mapem_item, ett_mapem);
        dissect_MAPEM_PDU(tvb, pinfo, mapem_tree);
    }
}
/*--- proto_register_mapem -------------------------------------------*/
void proto_register_mapem(void) {

  module_t *mapem_module;

  /* List of fields */
  static hf_register_info hf[] = {

#include "packet-mapem-hfarr.c"
  };

  /* List of subtrees */
  static gint *ett[] = {
                  &ett_mapem,
#include "packet-mapem-ettarr.c"
  };


  /* Register protocol */
  proto_mapem = proto_register_protocol(PNAME, PSNAME, PFNAME);
  register_dissector("mapem", dissect_mapem, proto_mapem);

  /* Register fields and subtrees */
  proto_register_field_array(proto_mapem, hf, array_length(hf));
  proto_register_subtree_array(ett, array_length(ett));

  /* Register CLASS dissector tables */
  dissect_messageframe_pdu_type_table = register_dissector_table("dsrc.dsrcmsgid", "DSRC DSRCmsgID", FT_UINT8, BASE_DEC);
  dissect_regionalextension_pdu_type_table = register_dissector_table("dsrc.regionid", "DSRC RegionId", FT_UINT16, BASE_DEC);

  /* Register preferences module */
  mapem_module = prefs_register_protocol(proto_mapem, proto_reg_handoff_mapem);

  /* Register a sample port preference   */
  prefs_register_uint_preference(mapem_module, "btp.dport", "BTP destination port",
				 "BTP destination port",
				 10, &global_mapem_port);

}


/*--- proto_reg_handoff_mapem ---------------------------------------*/
void
proto_reg_handoff_mapem(void)
{
  static gboolean initialized = FALSE;
  static dissector_handle_t mapem_handle;
  static guint16 mapem_port;

  if (!initialized) {
    mapem_handle = create_dissector_handle(dissect_mapem, proto_mapem);
    dissector_add_handle("btp.dport", mapem_handle);
    initialized = TRUE;

    dissector_add_uint("dsrc.dsrcmsgid", mapData, new_create_dissector_handle(dissect_MapData_PDU, proto_mapem));

//    dissector_add_uint("dsrc.dsrcmsgid", signalPhaseAndTimingMessage, new_create_dissector_handle(dissect_SPAT_PDU, proto_mapem));

//    dissector_add_uint("dsrc.dsrcmsgid", signalRequestMessage, new_create_dissector_handle(dissect_SignalRequestMessage_PDU, proto_mapem));

//    dissector_add_uint("dsrc.dsrcmsgid", signalStatusMessage, new_create_dissector_handle(dissect_SignalStatusMessage_PDU, proto_mapem));

  } else {
    if (mapem_port != 0) {
      dissector_delete_uint("btp.dport", mapem_port, mapem_handle);
    }
  }
  if (global_mapem_port != 0) {
    dissector_add_uint("btp.dport", global_mapem_port, mapem_handle);
  }
  mapem_port = global_mapem_port;
}

