/* packet-spatem.c
 * Routines for SPATEM packet dissection
 *
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <glib.h>
#include <epan/packet.h>
#include <epan/asn1.h>
#include <epan/dissectors/packet-per.h>
#include <epan/prefs.h>

#include <stdio.h>
#include <string.h>

#include "packet-spatem.h"

#define PNAME  "SPATEM"
#define PSNAME "SPATEM"
#define PFNAME "spatem"
#define SPATEM_PORT 2004    /* BTP port */

void proto_reg_handoff_spatem(void);

/* Initialize the protocol and registered fields */
static int proto_spatem = -1;

/* Dissector tables */
static int global_spatem_port = SPATEM_PORT;
/* Used to identify MessageFrame */
static guint8 Ref_DSRCmsgID;
/* Used to identify RegionalExtension */
static guint16 Ref_RegionId;

typedef enum _RefDSRCmsgID_enum {
  mapData                     = 18,
  signalPhaseAndTimingMessage = 19,
  signalRequestMessage        = 20,
  signalStatusMessage         = 30
} refDSRCmsgID_enum;

typedef enum _RefRegionID_enum {
  noRegion = 0,
  addGrpA  = 1,
  addGrpB  = 2,
  addGrpC  = 3
} RefRegionId_enum;

#include "packet-spatem-hf.c"

/* Initialize the subtree pointers */
static int ett_spatem = -1;

static dissector_table_t dissect_messageframe_pdu_type_table;
static dissector_table_t dissect_regionalextension_pdu_type_table;

static int dissect_messageframe_pdu_type(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_);
static int dissect_regionalextension_pdu_type(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_);
static int dissect_xxx_Container(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_);

#include "packet-spatem-ett.c"

#include "packet-spatem-fn.c"

#if defined(BUILD_WINDOWS)
guint32
dissect_per_UTF8String(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, int min_len, int max_len, gboolean has_extension)
{
  offset = dissect_per_octet_string(tvb, offset, actx, tree, hf_index, min_len, max_len, has_extension, NULL);
  return offset;
}
#else
extern guint32 dissect_per_UTF8String(tvbuff_t *tvb, guint32 offset, asn1_ctx_t *actx, proto_tree *tree, int hf_index, int min_len, int max_len, gboolean has_extension);
#endif

static int
dissect_xxx_Container(tvbuff_t *tvb _U_, int offset _U_, asn1_ctx_t *actx _U_, proto_tree *tree _U_, int hf_index _U_) {
  printf("### dissect_XXX_container: Unsupported\n");
  offset = dissect_per_constrained_integer(tvb, offset, actx, tree, hf_index,
                                                            0U, 32767U, NULL, FALSE);

  return offset;
}

static int dissect_messageframe_pdu_type(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_)
{
  return (dissector_try_uint_new(dissect_messageframe_pdu_type_table, Ref_DSRCmsgID, tvb, pinfo, tree, FALSE, NULL)) ? tvb_captured_length(tvb) : 0;
}

static int dissect_regionalextension_pdu_type(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_)
{
  return (dissector_try_uint_new(dissect_regionalextension_pdu_type_table, Ref_RegionId, tvb, pinfo, tree, FALSE, NULL)) ? tvb_captured_length(tvb) : 0;
}

static void
dissect_spatem(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
    proto_item      *spatem_item = NULL;
    proto_tree      *spatem_tree = NULL;

    /* make entry in the Protocol column on summary display */
    col_set_str(pinfo->cinfo, COL_PROTOCOL, PNAME);

    /* Clear out stuff in the info column */
    col_clear(pinfo->cinfo,COL_INFO);

    /* create the spatem protocol tree */
    if (tree) {
        spatem_item = proto_tree_add_item(tree, proto_spatem, tvb, 0, -1, FALSE);
        spatem_tree = proto_item_add_subtree(spatem_item, ett_spatem);
        dissect_SPATEM_PDU(tvb, pinfo, spatem_tree);
    }
}
/*--- proto_register_spatem -------------------------------------------*/
void proto_register_spatem(void) {

  module_t *spatem_module;

  /* List of fields */
  static hf_register_info hf[] = {

#include "packet-spatem-hfarr.c"
  };

  /* List of subtrees */
  static gint *ett[] = {
                  &ett_spatem,
#include "packet-spatem-ettarr.c"
  };


  /* Register protocol */
  proto_spatem = proto_register_protocol(PNAME, PSNAME, PFNAME);
  register_dissector("spatem", dissect_spatem, proto_spatem);

  /* Register fields and subtrees */
  proto_register_field_array(proto_spatem, hf, array_length(hf));
  proto_register_subtree_array(ett, array_length(ett));

  /* Register CLASS dissector tables */
  dissect_messageframe_pdu_type_table = register_dissector_table("dsrc.dsrcmsgid", "DSRC DSRCmsgID", FT_UINT8, BASE_DEC);
  dissect_regionalextension_pdu_type_table = register_dissector_table("dsrc.regionid", "DSRC RegionId", FT_UINT16, BASE_DEC);

  /* Register preferences module */
  spatem_module = prefs_register_protocol(proto_spatem, proto_reg_handoff_spatem);

  /* Register a sample port preference   */
  prefs_register_uint_preference(spatem_module, "btp.dport", "BTP destination port",
				 "BTP destination port",
				 10, &global_spatem_port);

}


/*--- proto_reg_handoff_spatem ---------------------------------------*/
void
proto_reg_handoff_spatem(void)
{
  static gboolean initialized = FALSE;
  static dissector_handle_t spatem_handle;
  static guint16 spatem_port;

  if (!initialized) {
    spatem_handle = create_dissector_handle(dissect_spatem, proto_spatem);
    dissector_add_handle("btp.dport", spatem_handle);
    initialized = TRUE;

//    dissector_add_uint("dsrc.dsrcmsgid", mapData, new_create_dissector_handle(dissect_MapData_PDU, proto_spatem));

    dissector_add_uint("dsrc.dsrcmsgid", signalPhaseAndTimingMessage, new_create_dissector_handle(dissect_SPAT_PDU, proto_spatem));

//    dissector_add_uint("dsrc.dsrcmsgid", signalRequestMessage, new_create_dissector_handle(dissect_SignalRequestMessage_PDU, proto_spatem));

//    dissector_add_uint("dsrc.dsrcmsgid", signalStatusMessage, new_create_dissector_handle(dissect_SignalStatusMessage_PDU, proto_spatem));

  } else {
    if (spatem_port != 0) {
      dissector_delete_uint("btp.dport", spatem_port, spatem_handle);
    }
  }
  if (global_spatem_port != 0) {
    dissector_add_uint("btp.dport", global_spatem_port, spatem_handle);
  }
  spatem_port = global_spatem_port;
}

